#ifndef __POINT3_H__
#define __POINT3_H__
#include "types.h"
#include <GL/gl.h>
#include <iostream>
#include <math.h>
#include "Vector.h"
#pragma pack(16)

/*!
\file Point3.h
\brief encapsulates a 3D point
\author Jonathan Macey
\version 1.0
\date 23/11/04
\warning Real x,y,z must be first data elements in the class as these are accessed by the [] operator
*/


/*! \todo lots of tidying up and code optimizations
\todo at some stage re-write to use Machine Code and MMX extentions
*/
/*! \class Point3 "include/Point3.h"
\breif simple Point3 class for OpenGL graphics, contains overloaded operators for
most math functions, including operators for manipulation by Vector and Matrix
relies on the fact that x,y and z are the first data elements of the class
*/


/* \namespace GraphicsLib all elements of the GraphicsLib are placed in the namespace GraphicsLib */
namespace GraphicsLib {
/*! class to encapsulate x,y,z points */

class Point3
{

/*! << extraction operator to prints out the point in the format [x,y,z]
   \param std::ostream & output the outstream class pointer
   \param const Point3 &s the point to print out 
*/
friend std::ostream& operator<<(std::ostream& output, const GraphicsLib::Point3& s);

/*! << extraction operator to prints out the point in the format [x,y,z] using a pointer to Point3
   \param std::ostream & output the outstream class pointer
   \param const Point3 *s the *point to print out 
*/

friend std::ostream& operator<<(std::ostream& output, const GraphicsLib::Point3* s);
/*! >> insertion operator to read in the point data x -> y -> z
   \param std::istream & input the instream class pointer
   \param const Point3 s the point to read in to
*/
friend std::istream& operator>>(std::istream& input,GraphicsLib::Point3 &s);
/*! >> insertion operator to read in the point data x -> y -> z into a Point3 *
   \param std::istream & input the instream class pointer
   \param const Point3 *s the point to read in to
*/

friend std::istream& operator>>(std::istream& input,GraphicsLib::Point3 *s);
public:
/*! \var Real x the x value of point */
 Real x;
/*! \var Real y the y value of point */
 Real y;
/*! \var Real z the z value of point */
 Real z;
/*! \fn inline void set(Real nx, Real ny, Real nz)
   this function set the point values
 * \param Real nx the new x value
 * \param Real ny the new y value
 * \param Real nz the new z value */
 inline void set(Real nx, Real ny, Real nz) {x = nx; y = ny; z = nz;}
 /*! \fn inline void set(const Point3 &p)
   this function set the point values based on a Point3 passed in 
  \param const Point3 &p the point to be copied*/

 inline void set(const Point3 &p){x = p.x; y = p.y; z = p.z;}
 /*! \fn inline void set(const Point3 *p)
   this function set the point values based on a Point3 * passed in 
  \param const Point3 *p the point to be copied*/

 inline void set(const Point3 *p){x = p->x; y = p->y; z = p->z;}
/*! \fn Point3(Real nx=0.0, Real ny=0.0, Real nz=0.0)
 	constructor using paramerer values
 * \param Real nx x value default =0.0
 * \param Real ny y value default =0.0
 * \param Real nz z value default =0.0
 */
 
 Point3(Real nx=0.0, Real ny=0.0, Real nz=0.0) : x(nx), y(ny), z(nz) {}
 /*! \fn Point3(const Point3& np)
 	copy constructor using another point
 * \param Point3& np the point to copy from
 */
 
 Point3(const Point3& np) : x(np.x), y(np.y), z(np.z) {}
 /*! \fn Point3(const Point3& np)
 	copy constructor using another point passed as a pointer
 * \param Point3* np the point to copy from
 */
 
 Point3(const Point3* np) : x(np->x), y(np->y), z(np->z) {}
 
 /*!\fn inline void build4tuple(Real v[])
 build a 4 tuple with w=1 for the 4th element
 * \param Real v[] the array to be filled 
 \brief this function is used for opengl commands  when they need a fv param
  */
 inline void build4tuple(Real v[])
  {// load 4-tuple  v[3] = 1 for homogeneous
   v[0] = x; v[1] = y; v[2] = z; v[3] = 1.0f;
  }
/*! math operator for adding two points
\param const Point3 &rhs the right hand side argument
\returns Point3 the result of this+rhs */
Point3 operator+(const Point3& rhs);
Point3 operator+(const Point3* rhs);

/*! math operator for adding point and a vector
\param const Vector &rhs the right hand side argument
\returns Point3 the result of this+rhs */
Point3 operator+(const Vector& rhs);
/*! math operator for adding a point to the current point
\param const Point3 &rhs the right hand side argument
*/
void operator+=(const Point3 &rhs);
/*! math operator for adding a vector to the current point
\param const Point3 &rhs the right hand side argument
*/
void operator+=(const Vector &rhs);
/*! math operator for subtracting two points
\param const Point3 &rhs the right hand side argument
\returns Vector the result of this-rhs */
Vector operator-(const Point3& rhs);
/*! math operator for subtracting a Vector from a point
\param const Vector &rhs the right hand side argument
\returns Point3 the result of this-rhs */
Point3 operator-(const Vector& rhs);
/*! math operator for subtracting a point from the current point
\param const Point3 &rhs the right hand side argument
*/
void operator-=(const Point3 &rhs);
/*! math operator for subtracting a Vector from the current point
\param const Vector &rhs the right hand side argument
*/
void operator-=(const Vector &rhs);
/*! math operator for multiplying a point by a Real scalar
\param Real &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/
Point3 operator*(Real rhs);
/*! math operator for multiplying the current point by a Real scalar
\param Real &rhs the right hand side argument
*/
void operator*=(Real rhs);
/*! math operator for multiplying a point by matrix
\param Matrix &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/
Point3 operator*(const Matrix &rhs);
/*! math operator for multiplying a point by a Vector
\param Vector &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/
Point3 operator*(const Vector &rhs);
/*! math operator for multiplying a point by a Point
\param Point3 &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/

Point3 operator* (const Point3 &rhs);
/*! math operator for dividing a Point by a Real scalar
\param Real &rhs the right hand side argument
\returns Point3 the result of this / rhs;
*/
Point3 operator/(Real rhs);
/*! relation operator == compare two points
\param const Point3 &rhs
\returns true if x==rhs.x && y==rhs.y && z==rhs.z else false
*/
bool operator==(const Point3& rhs);
/*! relation operator != compare two points
\param const Point3 &rhs
\returns true if x!=rhs.x && y!=rhs.y && z!=rhs.z else false
*/
bool operator!=(const Point3& rhs);
Real operator[](int i);
Point3& operator=(const Point3 &rhs);
Point3 operator-();


/*! use the point as a glVertex */
void Vertex(void);
/*! use glTranslate to move to the current point position */
void Translate(void);
/*! use the current point as a normal */
void Normal(void);
/*! Normalize the current point */
void normalize(void);
};
inline Point3 operator *(float k, const Point3 &v) 
{
return Point3(k*v.x, k*v.y, k*v.z);
}

}
  
#endif
