#ifndef __VECTOR_H_
#define __VECTOR_H_

#include "types.h"
#include <math.h>
#include <iostream>
#ifdef __USINGWINDOWS__
	#include <windows.h>
#endif

#include <GL/gl.h>

namespace GraphicsLib{
// pre-declare the matrix class
class Matrix;


/*! A general vector class for a 4d vector x,y,z,W
 * includes operator overloading for vector maths
 */

class Vector
{
friend std::ostream& operator<<(std::ostream& output, const GraphicsLib::Vector& s);

 public:
/*! x component of the vector */
 	Real x;
/*! y component of the vector */
	Real y;
/*! z component of the vector */
    Real z;
/*! w component of the vector 0= Vector 1=Point*/
	Real w;
/*! call glNormal with the x y and z components */
	void Normal(void);
/*! call glVertex3f with the x y and z components */
	void Vertex(void);
/*! call glTranslate with the x y and z components */
	void Translate(void);
/*! return this dotted with b
 * \param Vector b vector to dot current vector with
 * \returns Real the dot product */
	Real dot(Vector b); // return this dotted with b
/*! default constructor un-initialized */
	inline Vector() { x=y=z=w=0.0; } // un-initialized constructor
/*! sets the vector component from 3 values
 * \param Real xx the x component
 * \param Real yy the y component
 * \param Real zz the z component */
	void set(Real xx, Real yy, Real zz);
/*! sets the vector component from 3 values
 * \param Real xx the x component
 * \param Real yy the y component
 * \param Real zz the z component 
 * \param Real ww the w component
 * */

	void set(Real xx, Real yy, Real zz, Real ww);

/*! set from another  vector
 * \param Vector vin the vector to set from */
	void set(Vector vin);
/*! initialise the constructor from 3 or 4 Real
 * \param Real xx x value
 * \param Real yy y value
 * \param Real zz z value
 * \param Real ww=1.0f so we dont have to pass it
 */
	// initialize from 3 or 4 Reals
 	Vector(Real xx,Real yy,Real zz,Real ww=1.0f);

	// set value from 3 or 4 Reals

	// clears vector to (0,0,0,1);
/*!clears the vector to 0,0,0,1 */
	void null(void);
/*! [] operator to access the index component of the vector
 * \returns Real this[x] as a Real */

	// indexed component
	Real& operator[](GLint i) ;
/*! returns the length of the vector
 * \returns Real \f$\sqrt{x^2+y^2+z^2} \f$ */
	// length of the vector
	Real length(void);
/*! negate the vector (make all components the negative) */
	// inverts the vector
	void negate(void);
/*!Normalize the vector using
 * <BR> \f$x=x/\sqrt{x^2+y^2+z^2} \f$
 * <BR> \f$y=y/\sqrt{x^2+y^2+z^2} \f$
 * <BR> \f$z=z/\sqrt{x^2+y^2+z^2} \f$
 */
	// make vector length one
	void normalize(void);
/*!set the vector as the cross product from 2 other vectors
 * \param Vector &v1 the first vector
 * \param Vector &v2 the second vector
 */
	// set vector as cross product from 2 other
	void cross(const Vector& v1,const Vector& v2);
/*!Return the cross product of this cross with b
 *  \param Vector b the vector cross this with
 *  \return Vector the result of this cross b */
	Vector cross(const Vector b); //return this cross b
/*!+= operator add this to vector v
 * \param Vector &v vector to add */
	// operator vector+=vector
	void operator+=(const Vector& v);
/*! -= operator this-=v
 * \param Vector &v vector to subtract
 */
	// operator vector-=vector
	void operator-=(const Vector& v);
/*! *= this*=v
 * \param Vector &v value to mult by */
	// operator vector*=vector
	void operator*=(const Vector& v);
/*! this * i for each element
 * \param Real i the scalar to mult by
 * \returns Vector
 */
	Vector operator *(const Real i);

/*! /= this/=v
 * \param Vector &v the value to divide by */
	// operator vector/=vector
	void operator/=(const Vector& v);
/*! + operator add vevtor+vector
 * \param Vector &v the value to add
 * \returns Vector */
	// operator vector+vector
//	Vector operator+(Vector& v);
	Vector operator+(const Vector &v);

	bool operator==(const Vector &v);
	Vector operator/(Real v);


/*! - operator subtract vevtor-vector
 * \param Vector &v the value to sub
 * \returns Vector */

	// operator vector-vector
	Vector operator-(const Vector& v);
/*! * operator mult vevtor*vector
 * \param Vector &v the value to mult
 * \returns Vector */
	// operator vector*vector
//	Vector operator*(Vector const &v);

/*! * operator mult vevtor*vector
 * \param Vector v the value to mult
 * \returns Vector */
	// operator vector*vector
	Vector operator*(const Vector &v);
	
	
	
	/*! \ operator div vevtor\vector
 * \param Vector &v the value to div by
 * \returns Vector */

	// operator vector/vector
	Vector operator/(const Vector& v);


	GLdouble AngleBetween(const Vector &rhs)const;

	Vector Reverse(void);
	double inner(const Vector& vector); 

	Vector outer(const Vector& vector);


	inline Real LengthSquared(void) const 
	{ 
	return x * x+y * y+ z*z;
	}

	Vector operator*(const Matrix &m);


};

}
#endif
