
#ifndef __MATERIAL_H__
#define __MATERIAL_H__

#include "Colour.h"
#ifdef __USINGWINDOWS__
#include <windows.h>
#endif
#include <GL/gl.h>
#include <math.h>
#include "types.h"
/*! \enum STDMAT used to give offset into materials array for some standard matertial values */
namespace GraphicsLib
{
enum STDMAT { BLACKPLASTIC,BRASS,BRONZE,CHROME,COPPER,GOLD,PEWTER,SILVER,POLISHEDSILVER};
static const Real materials [9][10]=
{
{ 0.0,0.0,0.0,0.01,0.01,0.01,0.50,0.50,0.50,32.0}, //Black plastic
{ 0.329412,0.223529,0.027451,0.780392,0.568627,0.113725,0.992157,
  0.941176,0.807843,27.8974}, //Brass
{ 0.2125,0.1275,0.054,0.714,0.4284,0.18144,0.393548,0.271906,
  0.166721,25.6}, // Bronze
{ 0.25,0.25,0.25,0.4,0.4,0.4,0.774597,0.774597,0.774597,76.8}, //Chrome
{ 0.19125,0.0735,0.0225,0.7038,0.27048,0.0828,0.256777,0.137622,
  0.086014,12.8}, // Copper
{ 0.274725,0.1995,0.0745,0.75164,0.60648,0.22648,0.628281,
  0.555802,0.3666065,51.2}, // Gold
{ 0.10588,0.058824,0.113725,0.427451,0.470588,0.541176,
  0.3333,0.3333,0.521569,9.84615}, // Pewter
{ 0.19225,0.19225,0.19225,0.50754,0.50754,0.50754,0.508273,
  0.508273,0.508273,51.2}, // Silver
{ 0.23125,0.23125,0.23125,0.2775,0.2775,0.2775,0.77391,
  0.77391,0.77391,89.6} // PolishedSilver

};


class Material
{
public :
	/*! Colour ambient ambient component of the material */
	Colour ambient;
	/*! Colour diffuse diffuse component of the material */
	Colour diffuse;
	/*! Colour specular component of the material */
	Colour specular;
	/*! Colour emissive component of the material */
	Colour emissive;
    
    /*! the number of parameters for the textures */
    int numParams; // for textures
    /*! an array of texture co-ords */
    Real params[10]; // for textures
    /*! type for texture */
    int textureType;  // 0 for none, neg for solids, pos for images

	Real specularExponent, reflectivity, transparency, SpeedOfLight;
	Real specularFraction, surfaceRoughness,Priority,fDisableRefraction;
	/*! set the default material properties */
	void setDefault();
	/*! set the material properties based on another matherial
    \param Material m the material to use */
    void set(Material m);
	/*! set the material properties based on another matherial
    \param Material *m the material to use */
    void set(Material *m);
	/*! change the material to one of the standard materials
    \param STDMAT mat the material to use */
    void Change(STDMAT mat);
	/*! Use this material */
    void Use();
	/*! use this material with a transparency value
    \param Real transp the transparency value to use 1 = solid 0 = full transp */
    void Use(Real transp);
	/*! change the ambient material component
	 * \param Colour amb value to change to */
	void setAmbient(Colour amb){ambient=amb;}
	/*! change the diffuse material component
	 * \param Colour diff value to change to */
	void setDiffuse(Colour diff){diffuse=diff;}
	/*! change the specular material component
	 * \param Colour spec value to change to */

	void setSpecular(Colour spec){specular=spec;}
	/*! change the Emissive material component
	 * \param Colour emissive value to change to */

	void setEmissive(Colour em){emissive=em;}
 	/*! get the current ambient component
	 * \returns Colour ambient value*/

	Colour getAmbient(void){return ambient;}
	/*! get the current specular component
	 * \returns Colour specular value */
	Colour getSpecular(void){return specular;}
	/*! gets the diffuse component
	 * \returns Colour diffuse */
	Colour getDiffuse(void){return diffuse;}
	/*! gets the current emissive component
	 * \returns Colour emissive  */
	Colour getEmissive(void){return emissive;}
	/*!Set the global transparency for a material NOTE: this will have
	 * 	 * no effect unless GL_BLEND has been enabled
	 * 	 	 * \param Real transp the value for transparency 0.0 =invisible 1.0 = SOLID */

	void SetTransparency(Real transp){transparency=transp;}
   /*! returns the transparency value for the object
	* \returns Real transparency */
	Real GetTransparency(void){return transparency;}
	/*! Constructor for material
    \param Colour amb the ambient colour
    \param Colour dif the diffuse colour
    \param Colour spec the specular colour
    \param Colour emis the emimissive colour */
	Material(Colour amb, Colour dif, Colour spec, Colour emis)
	{ ambient=amb; diffuse=dif; specular=spec, emissive=emis; }
	/*! Constructor using a STDMAT for the coulour
    \param STDMAT mat */
    Material(STDMAT mat);
    Material(int mat);
    Material(std::string fname);
    void Load(std::string fname);
	/*! default constructor */
    Material(){;}
}; // end of Material



}

#endif // MATERIAL
