#ifndef __OBJLOADER_H_
#define __OBJLOADER_H_

#ifdef __USINGWINDOWS__
	#include <windows.h>
#endif
#include <iostream>
#include <fstream>
#include "Point3.h"
#include "Image.h"
#include <math.h>
#include "types.h"
/*! \class ObjLoader.h <BR>
	Author Jonathan Macey <BR>
    Version 2.0 <BR>
    Date Last Revision 17/08/03 <BR>
    Linkages : Point3.h <BR>
	PreProcessor : If using windows define __USINGWINDOWS__ flag
                   For debug version define __DEBUG_ so it doesn't conflict with the windows one <BR>
    Description : <BR>
	used to load in an alias wave front obj format file and draw using open gl
    \todo add Texture support to the file parser
    \todo add material support to the file parser
*/

/*! \class OBjFace simple class used to encapsulate a single face of an Obj file */
namespace GraphicsLib {
class ObjFace
{
public :
	/*! the number of vertices in the face */
    unsigned int NumVerts;
	/*! The vertices index */
    unsigned long int Vert[18];
	/*! The texture co-ord index */
    unsigned long int Tex[18];
	/*! the normal index */
    unsigned long int Norm[18];
   /*! debug flag to turn face on and off */
   bool inface;
};

class ObjLoader
{

public :
	/*! obj file tokens */
    enum ObjToken{OVERTEX,ONORMAL,OTEXTURE,OFACE,OUNKNOWN};
    /*! The number of vertices in the object */
    unsigned long int nVerts;
	/*! The number of normals in the object */
    unsigned long int nNorm;
	/*! the number of texture co-ordinates in the object */
    unsigned long int nTex;
	/*! the number of faces in the object */
    unsigned long int nFaces;
	/*! Pointer to the Vertex list */
    Point3 *Verts;
	/*! Pointer to the Normal List */
    Point3 *Norm;
	/*! Pointer to the Textur co-ord list (note that only x and y are used) */
    Point3 *Tex;
	/*! Pointer to the Face list */
    ObjFace *Face;
	/*! Method to load the file in
    \param char *fname the name of the obj file to load */
    void Load(char *fname);
	/*! default constructor */
    ObjLoader();
    /*! constructor to load an objfile as a parameter
    \param char *fname the name of the obj file to load */
    ObjLoader(char *fname);
	/*! constructor to load an objfile as a parameter
    \param char *fname the name of the obj file to load 
    \param char *fname2 the name of the texture file */
    
    ObjLoader(char *fname,char *fname2);
    /*! constructor to load an objfile as a parameter
    \param char *fname the name of the obj file to load 
    \param bool i
    */
    ObjLoader(char *fname,bool i);
	/*! destructor */
    ~ObjLoader();
	/*! Draw method to draw the obj */
    void Draw(void);
	/*! draw method to draw the obj as a display list. The display list is automatically allocated
    at construction time and this is the faster of the two draw methods */
    void DrawDisplayList(void);
	void LoadTexture(char *Fname2);
	Real GetHeight(Point3 p);
	void Scale(Real x, Real y, Real z);
	void ResetInFace(void);
private :
	/*! the current vertex index in the vertex list */
    unsigned long int currVert;
	/*! the current normal index in the normal list */
    unsigned long int currNorm;
	/*! the current tex co-ord index in the tex list */
     unsigned long int currTex;
	/*! the current face index in the face list */
    unsigned long int currFace;
    /*! method to parse a vertex record and add a vertex to the vertex list
    \param char *str the string to parse */
	void ParseVertex(char *str);
	/*! method to parse a normal record and add a vertex to the vertex list
    \param char *str the string to parse */
    void ParseNormal(char *str);
	/*! method to parse a textur record and add a vertex to the vertex list
    \param char *str the string to parse */
    void ParseTex(char *str);
	/*! method to parse a face record and add a vertex to the vertex list
    \param char *str the string to parse */
    void ParseFace(char *str);
    /*! method to parse the line record and determine what type it is
    \returns ObjToken the token found */
    ObjToken ParseString(char *str);
	/*! the display list index created from glCreateLists */
    GLuint ListIndex;
	/*! method to setup the display list */
    void CreateDisplayList(void);
	/*! the Image used for the texture */
	Image *Texture;
	/*! The name of the texture for the texture generation */
	unsigned int Tname;
};

}


#endif
