#include <iostream>
#include "Camera.h"

using namespace std;
using namespace GraphicsLib;
// a lot of this stuff is from the HILL book Computer Graphics with OpenGL 2nd Ed Prentice Hall
// a very good book
/*!
 setDefaultCamera sets the camera shape to a good start
 This is used in the default constructor so any camera will
 work

 */

void Camera :: setDefaultCamera(void)
{ // make default camera
	setShape(45.0f, 640/(Real)480, 0.1f, 200.0f); // good default values here
	set(5.0, 5.0, 5.0, 0.0, 0.0, 0.0, 0, 1, 0);
}
//<<<<<<<<<<<<<<<<<<< setDisplay >>>>>>>>>>>>
/*!
 Set display tells the camera the screen size
 \param GLint numRows indicates the number of rows for the screen size
 \param GLint numCols indicates the number of col for the screen
 */

void Camera :: setDisplay(GLint numRows, GLint numCols)
{ // tell camera screen size
	nRows = numRows; 	nCols = numCols;
}
/*! set
 set sets the camera's Eye look and up vectors
 we do this by passing in the points and then
 normalizing the values
 Finally we set this to be the default Model view matrix
 \param Point3 Eye the eye position
 \param Point3 look Where we are looking
 \param Vector up indicate which axis is the up direction
 */


void Camera :: set(Point3 eye, Point3 look, Vector up)
{ // make u, v, n vectors
	Eye=eye;
	n.set(Eye.x - look.x, Eye.y - look.y,Eye.z - look.z);
	u.set(up.cross(n));
	v.set(n.cross(u));
	u.normalize(); v.normalize(); n.normalize();
	Look=look;
	UP=up;
	setModelViewMatrix();
}
Camera::Camera(Point3 eye, Point3 look, Vector Up)
{
setDefaultCamera();
set(eye,look,Up);
}
/*! This set the camera using individual Reals for the
    paramerers (EYE LOOK and UP) see the previous version of set
	\param Real ex,ey,ez individual eye components
	\param Real lx,ly,lz individual look components
	\param Real upx,upy,upz individual up vector values
*/

void Camera :: set(Real ex, Real ey, Real ez, Real lx, Real ly, Real lz, Real upx, Real upy, Real upz)
{
	Point3 e(ex, ey, ez);
	Point3 look(lx, ly, lz);
	Vector Up(upx, upy, upz);
	set(e,look, Up);
    
}

void Camera :: setModelViewMatrix(void)
{ // load modelview matrix with existing camera values

   Real m[16];
   Vector eVec(Eye.x, Eye.y, Eye.z);

   m[0] = u.x; m[4] = u.y; m[8] = u.z; m[12] = -eVec.dot(u);
   m[1] = v.x; m[5] = v.y; m[9] = v.z; m[13] = -eVec.dot(v);
   m[2] = n.x; m[6] = n.y; m[10] = n.z; m[14] = -eVec.dot(n);
   m[3] = 0; m[7] = 0; m[11] = 0; m[15] = 1.0;

   glMatrixMode(GL_MODELVIEW);
   glLoadMatrixf(m);
}

void Camera:: setProjectionMatrix(void)
{
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(viewAngle, aspect, nearDist, farDist);
   glMatrixMode(GL_MODELVIEW); //set its mode back again
}
//<<<<<<<<<<<<<<<<<<<<<<< setShape >>>>>>>>>>>>>>>>
/*!
 * \param Real vAngle the view angle of the view pyramid
 * \param Real asp the aspect ratio of the camera
 * \param Real nr the near view plane of the camera
 * \param Real fr the far view plane of the camera
 */

void Camera :: setShape(Real vAngle, Real asp, Real nr, Real fr)
{ // load projection matrix and camera values

   viewAngle = vAngle;

   //viewangle in degrees - must be < 180

   aspect = asp;
   nearDist = nr;
   farDist = fr;
   H = nearDist * tan(3.14159265 * viewAngle / (2.0 * 180));
   W = aspect * H;
   setProjectionMatrix();
}
//<<<<<<<<<<<<<<<<<<<<<<<<<<<< setAspect >>>>>>>>>>>>>>>>>>>>>>
/*! Sets the camera aspect ratio using the default class values and
  an aspect ration passed in
  \param Real asp the aspect ratio for the camera
*/

void Camera:: setAspect(Real asp)
{
aspect = asp; W = aspect * H;
setShape(viewAngle,aspect,nearDist,farDist);
}

void Camera :: setViewAngle(Real Angle)
{
setShape(Angle,aspect,nearDist,farDist);
}

//<<<<<<<<<<<<<<<<<<<<<<< getShape >>>>>>>>>>>>>>>>
/*! Returns the shape of the camera to the parameters passed in
 * \param Real &vAngle the angle of the view pyramid
 * \param Real &asp the aspect ratio
 * \param Real &nr near view plane
 * \param Real &fr far view plane
 */
void Camera :: getShape(Real& vAngle, Real& asp, Real& nr, Real& fr)
{   
	vAngle = viewAngle;
	asp = aspect;
	nr = nearDist;
	fr = farDist;
}
//<<<<<<<<<<<<<<<<<<<<<< slide >>>>>>>>>>>>>>>>>>>>>>..
/*! \param GLdouble du the ammount to slide along the u vector
 *  \param GLdouble dv the ammount to slide along the v vector
 *  \param GLdouble dn the ammount to slide along the n vector
 */
void Camera :: slide(GLdouble du, GLdouble dv, GLdouble dn)
{ // slide both eye and lookAt by amount du * u + dv * v + dn * n;
   Eye.x += du * u.x + dv * v.x + dn * n.x;
   Eye.y += du * u.y + dv * v.y + dn * n.y;
   Eye.z += du * u.z + dv * v.z + dn * n.z;
   setModelViewMatrix();
}

void Camera :: move(double dx, double dy, double dz)
{
    // simply add the translation to the current eye point
   Eye.x += dx;
   Eye.y += dy;
   Eye.z += dz;

//cerr <<"Method not Implimented "<<__FILE__<<" "<<__LINE__<<endl;

setModelViewMatrix();
}

void Camera :: moveBoth(double dx, double dy, double dz)
{
   Look.x += dx;
   Look.y += dy;
   Look.z += dz;
   Eye.x += dx;
   Eye.y += dy;
   Eye.z += dz;

   setModelViewMatrix();
}

//<<<<<<<<<<<<<<<<<<<<<<<< rotAxes >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/*! \param Vector &a the first vector for rotation axis
 *  \param Vector &b the second vector for ratation axis
 *  \param Real angle the angle in degrees to rotate
 *         this may need converting into radian
 *
 */
void Camera:: rotAxes(Vector& a, Vector& b, Real angle)
{ // rotate orthogonal vectors a (like x axis) and b(like y axia) through angle degrees
   Real ang = M_PI/180*angle;
   Real C = cos(ang), S = sin(ang);
   Vector t( C * a.x + S * b.x, C * a.y + S * b.y, C * a.z + S * b.z);
   //tmp for a vector
   b.set(-S * a.x + C * b.x, -S * a.y + C * b.y, -S * a.z + C * b.z);
   a.set(t.x, t.y, t.z); // put tmp into a
}
//<<<<<<<<<<<<<<<<<<<<<<<< roll >>>>>>>>>>>>>>>>>>>>>>>>>>>

/*! \param Real angle number of degrees to roll
 * */

void Camera :: roll(Real angle)
{
	rollAngle=angle;
    rotAxes(u, v, -angle); setModelViewMatrix();
}
//<<<<<<<<<<<<<<<<<<<<<<<< pitch >>>>>>>>>>>>>>>>>>>>>>>>>
/*!  \param Real angle number of degrees to pitch
 * */

void Camera :: pitch(Real angle)
{
    pitchAngle=angle;
	rotAxes(n, v, angle); setModelViewMatrix();
}
//<<<<<<<<<<<<<<<<<<<<<<<<< yaw >>>>>>>>>>>>>>>>>>>>>>>>
/*!
 *  \param Real angle number of degrees to yaw
 *
 * */

void Camera :: yaw(Real angle)
{
	yawAngle=angle;
	rotAxes(u, n, angle); setModelViewMatrix();
}

void Camera::moveEye(double dx, double dy, double dz)
{
   Eye.x += dx;
   Eye.y += dy;
   Eye.z += dz;

   setModelViewMatrix();
}
void Camera::moveLook(double dx, double dy, double dz)
{
   Look.x += dx;
   Look.y += dy;
   Look.z += dz;

   setModelViewMatrix();
}
void Camera::Use(void)
{
setProjectionMatrix();
setModelViewMatrix();
}

void Camera :: normalisedYaw(Real angle)
{
    // build a rotation matrix around the y axis
    Matrix mat;
    mat.Identity();
    mat.RotateY(angle);
    //multiply all three local coord vectors by the matrix
    u = u * mat;
    v = v * mat;
    n = n * mat;

    // reset the modelview matrix
    setModelViewMatrix();
}
ostream& operator<<(ostream& output, const Camera & c)
{
 return output<<"Eye"<<c.Eye<<" Look"<<c.Look<<" Up"<<c.UP;
}
