//---------------------------------------------------------------------------

#include "CameraController.h"
#include <stdio.h>
using namespace GraphicsLib;
//---------------------------------------------------------------------------
CameraController::CameraController(Camera *cam)
{
    Cam = cam;

    Loaded = false;
    Running = false;
    SequenceTime = 0;
    Events = NULL;
    CurrentEvent = NULL;

    Pitch = 0;
    Yaw = 0;
    Roll = 0;
    X = 0;
    Y = 0;
    Z = 0;
}

//---------------------------------------------------------------------------
CameraController::~CameraController()
{
    Clear();
}

//---------------------------------------------------------------------------
void CameraController::Clear()
{
     // delete linked list of events
    if(Loaded)
    {
        CameraControlEvent *ptr = Events;
        CameraControlEvent *next;

        do
        {
            next = ptr->Next;
            delete ptr;
            ptr = next;

        }while (ptr);

        Loaded = false;
        Running = false;
        CurrentEvent = Events = NULL;
    }
}

//---------------------------------------------------------------------------
void CameraController::LoadControlScript(char *filename)
{
    // load from file
    if (Loaded)
        Clear();


    FILE *f;
    char buffer[128];
    f = fopen(filename, "r"); // opentext file for reading
    if (f == 0)
        // not loaded
        Loaded = false;
    else
    {
        int loop = 0;
        // parse data
        while(fgets(buffer, 128, f))
        {
            // format is :
            // TIME <tab> PITCH,YAW,ROLL <tab> X, Y, Z\n
            CameraControlEvent *event = new CameraControlEvent;
            if(loop==0)
                Events = event; // set pointer to first
            else
                CurrentEvent->Next = event; // set linked list

            CurrentEvent = event; // store pointer to this event for next iteration

            int items = sscanf(buffer, "%i\t%f,%f,%f\t%f,%f,%f", &event->EventTime,
                               &event->Pitch, &event->Yaw, &event->Roll,
                               &event->X, &event->Y, &event->Z);
            if(items != 7)
                // error reading line so end loop
                break;
            else
                loop++;

        }

        // if we have at least one event then we have sucessfully loaded
        if(Events)
            Loaded = true;

        // reset pointers
        Reset();
    }


    // clean up
    fclose(f);



}

//---------------------------------------------------------------------------
void CameraController::Update(int Elapsed)
{
    if(!Loaded || !CurrentEvent || !Running || !Elapsed >0)
        return;

    SequenceTime += Elapsed;

    // look to see if the current event has elapsed
    if(SequenceTime >= CurrentEvent->EventTime)
    {
        // check that we have moved the required amount

        Cam->pitch(CurrentEvent->Pitch - Pitch);
        Cam->normalisedYaw(CurrentEvent->Yaw - Yaw);
        Cam->roll(CurrentEvent->Roll - Roll);
        Cam->move((CurrentEvent->X - X), (CurrentEvent->Y - Y), (CurrentEvent->Z - Z));

        // move onto next event
        if(CurrentEvent->Next)
        {
            CurrentEvent = CurrentEvent->Next;
            Pitch = 0;
            Yaw = 0;
            Roll = 0;
            X = 0;
            Y = 0;
            Z = 0;

        }
        else
        {
            // end sequence
            Running = false;
            CurrentEvent= NULL;
        }


    }
    else
    {
        // if not interpolate between current values and event values

        // get time left (including this update) for event
        int time_left = CurrentEvent->EventTime - (SequenceTime-Elapsed);

        // get number of (estimated) updates left in this event
        Real updatesLeft = ((Real)time_left) / ((Real)Elapsed);

        if(updatesLeft >0.0)
        {

            // add on values
            Real pitch = (CurrentEvent->Pitch - Pitch) / updatesLeft;
            Real yaw = (CurrentEvent->Yaw - Yaw) / updatesLeft;
            Real roll = (CurrentEvent->Roll - Roll) / updatesLeft;
            Real dx = (CurrentEvent->X - X) / updatesLeft;
            Real dy = (CurrentEvent->Y - Y) / updatesLeft;
            Real dz = (CurrentEvent->Z - Z) / updatesLeft;

            // Move Camera
            Cam->pitch(pitch);
            Cam->normalisedYaw(yaw);    // more realistic (human) movement
            Cam->roll(roll);
            Cam->move(dx, dy, dz);

            // add to internal values
            Pitch += pitch;
            Yaw += yaw;
            Roll += roll;
            X+=dx, Y+=dy, Z+= dz;
        }
    }
}

//---------------------------------------------------------------------------
void CameraController::Start()
{
    Running = true;
}

//---------------------------------------------------------------------------
void CameraController::Stop()
{
    Running = false;
}

//---------------------------------------------------------------------------
void CameraController::Reset()
{
    Pitch = 0;
    Yaw = 0;
    Roll = 0;
    X = 0;
    Y = 0;
    Z = 0;

    SequenceTime = 0;

    if(Events)
        CurrentEvent = Events;
}
//---------------------------------------------------------------------------
